<?php

class Controller_Api_V0_Teacher extends Controller_Handler_Api
{
    private function makeErrorMsg($data, $msg) {
        $dt = print_r($data, true);
        $res = 'Невозможно загрузить/обновить преподавателя. '.$msg.":\n ".$dt;
        return $res;
    }

    private function getDepIDByName($name) {
        try {
            $id = Model_Faculties::getDepartmentIdByName($name);
        }
        catch (InvalidArgumentException $e) {
            return NULL;
        }
        return $id;
    }

    private function getDepIDByExternalID($departmentExternalID) {
        try {
            $id = Model_Faculties::getDepartmentIdByExternalID($departmentExternalID);
        }
        catch (InvalidArgumentException $e) {
            return NULL;
        }
        return $id;
    }

    private function normalizeTeacherData($data) {
        if ( !isset($data->id) && isset($data->externalID) ) {
            $ID = Model_Teacher::withExternalID($data->externalID);
            $data->id = $ID;
        }

        if( !isset($data->jobPositionName) ) {
            $this->badRequestError( $this->makeErrorMsg($data, 'Не указана должность.') );
        }

        if ( !isset($data->departmentID) ) {
            if (  isset($data->departmentExternalID) ) {
                $depid = $this->getDepIDByExternalID($data->departmentExternalID);
                if ( $depid === NULL ) {
                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка коде справочника подразделения.') );
                }
                $data->departmentID = $depid;
            } elseif (  isset($data->departmentName) ) {
                $depid = $this->getDepIDByName($data->departmentName);
                if ( $depid === NULL ) {
                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка в имени подразделения.') );
                }
                $data->departmentID = $depid;
            } else {
                $this->badRequestError( $this->makeErrorMsg($data, 'Не указано подразделение.'));
            }
        }

        echo $data->firstName.' '.$data->secondName.' '.$data->lastName.' $data->id: '.$data->id."\n";
        if ( !isset($data->id) || ($data->id == -1) ) {
            echo "-- Try to find by name !\n";
            $foundedList = Model_Teachers::search([$data->firstName, $data->secondName, $data->lastName], 0, 0);
            if ( count($foundedList) === 1 ) {
                echo '--- Founded by name id: '.$foundedList[0]['ID']."!\n";
                $data->id = $foundedList[0]['ID'];
            }
        }

        if ( !isset($data->jobPositionID) && isset($data->jobPositionName) ) {
            $jpID = Model_Faculties::getJobPositionIdByName($data->jobPositionName);
            if ( $jpID === NULL ) {
                $jpID = Model_Faculties::createJobPosition($data->jobPositionName);
            }
            $data->jobPositionID = $jpID;
        } elseif( !isset($data->jobPositionName) ) {
            $this->badRequestError( $this->makeErrorMsg($data, 'Не указана должность.') );
        }

        switch ($data->status) {
            case 'Работает':
                $data->status = true;
                break;
            case 'Уволен':
                $data->status = false;
                break;
            default:
                $this->badRequestError( $this->makeErrorMsg($data, 'Указан некорректный статус.') );
        }

        return $data;
    }

    private function createTeacher($data) {
        try {
            $teacher = Model_Teacher::make()
                ->jobPosition($data->jobPositionID)
                ->department($data->departmentID)
                ->firstName($data->firstName)
                ->secondName($data->secondName)
                ->lastName($data->lastName);

            if ( isset($data->externalID) ) {
                $teacher = $teacher->externalID($data->externalID);
            }
            if ( isset($data->inila) ) {
                $teacher = $teacher->inila($data->inila);
            }
            $teacher = $teacher->create();
        } catch (InvalidArgumentException $e) {
            $this->badRequestError( $this->makeErrorMsg($data, $e->getMessage()) );
        }
        return ['ID' => $teacher->ID
                , 'lastName' => $data->lastName
                , 'firstName' => $data->firstName
                , 'secondName' => $data->secondName];
    }

    private function updateTeacher($data) {
        try {
            Model_Teacher::with($data->id)->changeInfo( $data->externalID,
                $data->lastName, $data->firstName, $data->secondName,
                $data->jobPositionID, $data->departmentID, $data->status
            );
        } catch (InvalidArgumentException $e) {
            $this->badRequestError( $this->makeErrorMsg($data, $e->getMessage()) );
        }
    }

    /**
     * @api {put} api/v0/teacher Add new teacher(s)
     * @apiName Add new teacher
     * @apiGroup Teacher
     * @apiVersion 0.1.0
     * @apiParam {String} token Api-токен
     * @apiParam {String} [batch] Флаг символизирующий о пакетной загрузке
     * @apiParam {String} lastName Фамилия преподавателя
     * @apiParam {String} firstName Имя преподавателя
     * @apiParam {String} [secondName] Отчество преподавателя
     * @apiParam {String} [departmentName] Название подразделения. Не обязателен, если представлен departmentID
     * @apiParam {String} [departmentID] Идентификатор подразделения. Не обязателен, если представлен departmentName
     * @apiParam {String} [departmentExternalID] Код справочника из 1С:Университет. Не обязателен, если представлен departmentID
     * @apiParam {String} [jobPositionName] Название должности. Не обязателен, если представлен jobPositionID
     * @apiParam {String} [jobPositionID] Идентификатор должности. Не обязателен, если представлен jobPositionName
     * @apiParam {String=Уволен,Работает} status Статус преподавателя
     */
    public function action_put_index() {
        $res = [];
        try {
            if ( $this->request->query('batch') !== NULL ) {
                $data = json_decode($this->request->body());

                foreach ($data as $item) {
                    try
                    {
                        $data = $this->normalizeTeacherData($item);
                        if ( $data->status ) {
                            $res[] = $this->createTeacher($data);
                        }
                    }
                    catch (Exception  $e)
                    {
                        echo 'Yet another exception:';
                        print_r($e->getMessage());
                    }
                }
            } else {
                $data = [ 'firstName' => $this->request->query('firstName')
                    , 'secondName' => $this->request->query('secondName')
                    , 'lastName' => $this->request->query('lastName')
                    , 'departmentName' => $this->request->query('departmentName')
                    , 'departmentID' => $this->request->query('departmentID')
                    , 'departmentExternalID' => $this->request->query('departmentExternalID')
                    , 'jobPositionName' => $this->request->query('jobPositionName')
                    , 'jobPositionID' => $this->request->query('jobPositionID')
                    , 'status' => $this->request->query('status')];

                $data = $this->normalizeTeacherData((object)$data);
                if ( $data->status ) {
                    $res[] = $this->createTeacher($data);
                }
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }

    /**
     * @api {put} api/v0/teacher/WithID Update teacher(s) with particular inner ID
     * @apiName Update teacher with inner id
     * @apiGroup Teacher
     * @apiVersion 0.1.1
     * @apiParam {String} token Api-токен
     * @apiParam {String} [batch] Флаг символизирующий о пакетной загрузке
     * @apiParam {Int} id Идентификатор преподавателя
     * @apiParam {String} lastName Фамилия преподавателя
     * @apiParam {String} firstName Имя преподавателя
     * @apiParam {String} [secondName] Отчество преподавателя
     * @apiParam {String} [departmentName] Название подразделения. Не обязателен, если представлен departmentID
     * @apiParam {String} [departmentID] Идентификатор подразделения. Не обязателен, если представлен departmentName
     * @apiParam {String} [departmentExternalID] Код справочника из 1С:Университет. Не обязателен, если представлен departmentID
     * @apiParam {String} [jobPositionName] Название должности. Не обязателен, если представлен jobPositionID
     * @apiParam {String} [jobPositionID] Идентификатор должности. Не обязателен, если представлен jobPositionName
     * @apiParam {String=Уволен,Работает} status Статус преподавателя
     */
    public function action_put_WithID() {
        try {
            if ( $this->request->query('batch') !== NULL ) {
                $data = json_decode($this->request->body());

                foreach ($data as $item) {
                    $data = $this->normalizeTeacherData($item);
                    $this->updateTeacher($data);
                }
            } else {
                $data = [ 'id' => $this->request->query('id')
                    , 'firstName' => $this->request->query('firstName')
                    , 'secondName' => $this->request->query('secondName')
                    , 'lastName' => $this->request->query('lastName')
                    , 'departmentName' => $this->request->query('departmentName')
                    , 'departmentID' => $this->request->query('departmentID')
                    , 'departmentExternalID' => $this->request->query('departmentExternalID')
                    , 'jobPositionName' => $this->request->query('jobPositionName')
                    , 'jobPositionID' => $this->request->query('jobPositionID')
                    , 'status' => $this->request->query('status')];

                $data = $this->normalizeTeacherData((object)$data);
                $this->updateTeacher($data);
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return "Success";
    }

    /**
     * @api {put} api/v0/teacher/WithExternalID Add or update teacher(s) with particular 1C ID
     * @apiName Update teacher with external id
     * @apiGroup Teacher
     * @apiVersion 0.1.1
     * @apiParam {String} token Api-токен
     * @apiParam {String} [batch] Флаг символизирующий о пакетной загрузке
     * @apiParam {Int}    externalID Код справочника для этого преподавателя из 1С:Университет
     * @apiParam {String} lastName Фамилия преподавателя
     * @apiParam {String} firstName Имя преподавателя
     * @apiParam {String} [secondName] Отчество преподавателя
     * @apiParam {String} [inila] Хэш СНИЛС преподавателя (для авторизации через OpenID.sfedu.ru)
     * @apiParam {String} [departmentName] Название подразделения. Не обязателен, если представлен departmentID
     * @apiParam {String} [departmentID] Идентификатор подразделения. Не обязателен, если представлен departmentName
     * @apiParam {String} [departmentExternalID] Код справочника из 1С:Университет. Не обязателен, если представлен departmentID
     * @apiParam {String} [jobPositionName] Название должности. Не обязателен, если представлен jobPositionID
     * @apiParam {String} [jobPositionID] Идентификатор должности. Не обязателен, если представлен jobPositionName
     * @apiParam {String=Уволен,Работает} status Статус преподавателя
     */
    public function action_put_WithExternalID() {
        try {
            if ( $this->request->query('batch') !== NULL ) {
                $data = json_decode($this->request->body());

                foreach ($data as $item) {
                    try
                    {
                        $data = $this->normalizeTeacherData($item);
                        echo "== And the ID is ".$data->id."!\n";
                        if ( $data->id == -1 )
                        {
                            echo "==== Create!\n";
                            $this->createTeacher($data);
                        }
                        else
                        {
                            echo "==== Update!\n";
                            $this->updateTeacher($data);
                        }
                    }
                    catch (Exception  $e)
                    {
                        echo 'Yet another exception:';
                        print_r($e->getMessage());
                    }
                }
            } else {
                $data = [ 'externalID' => $this->request->query('externalID')
                    , 'firstName' => $this->request->query('firstName')
                    , 'secondName' => $this->request->query('secondName')
                    , 'lastName' => $this->request->query('lastName')
                    , 'inila' => $this->request->query('inila')
                    , 'departmentName' => $this->request->query('departmentName')
                    , 'departmentID' => $this->request->query('departmentID')
                    , 'departmentExternalID' => $this->request->query('departmentExternalID')
                    , 'jobPositionName' => $this->request->query('jobPositionName')
                    , 'jobPositionID' => $this->request->query('jobPositionID')
                    , 'status' => $this->request->query('status')];

                $data = $this->normalizeTeacherData((object)$data);
                if ( $data->id == -1 )
                {
                    if ( $data->status ) {
                        $this->createTeacher($data);
                    }
                }
                else
                {
                    $this->updateTeacher($data);
                }
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return "Success";
    }
}
